using Nuwanda.Common;
using System.ComponentModel;

namespace Nuwanda.TfsSpotlight.Shell.Data
{
    /// <summary>A base class for all strongly typed bindable collections</summary>
    /// <typeparam name="T">Item type (IBaseEntity constraint)</typeparam>
    [Author("Bartomiej Legid")]
    public class BaseEntityCollection<T> : BindingList<T> where T : IBaseEntity
    {
        protected bool _isDirty;
        /// <summary>Checks whether there are elements in the collection that are dirty</summary>
        public bool IsDirty
        {
            get
            {
                foreach (var entity in this)
                {
                    if (entity.IsDirty)
                        return true;
                }

                return this._isDirty;
            }
        }

        public virtual bool IsValid
        {
            get
            {
                foreach (var entity in this)
                {
                    if (!entity.IsValid)
                        return false;
                }

                return true;
            }
        }

        public BaseEntityCollection()
        {
            this._isDirty = false;
        }

        /// <summary>Sets the collection as old (= not dirty)</summary>
        public void MarkOld()
        {
            foreach (var entity in this)
            {
                entity.MarkOld();
            }

            this._isDirty = false;
        }

        protected override void OnListChanged(ListChangedEventArgs e)
        {
            this._isDirty = e.ListChangedType != ListChangedType.ItemChanged;

            base.OnListChanged(e);
        }
    }
}