using Nuwanda.Common;
using System.Windows.Forms;

namespace Nuwanda.TfsSpotlight.Shell.Proxies.Interfaces
{
    /// <summary>Describes a presenter that drives each tab</summary>
    [Author("Bartomiej Legid")]
    public interface ITabPresenter
    {
        /// <summary>A control to place on a tab</summary>
        Control Control { get; }

        /// <summary>Tells whether the tab is working</summary>
        bool IsWorking { get; }

        /// <summary>A handler for tab activation</summary>
        void HandleActivation();

        /// <summary>A handler for tab deactivation</summary>
        void HandleDeactivation();

        /// <summary>A handler for tab remove</summary>
        void HandleClose();

        /// <summary>A handler for tab refresh</summary>
        void HandleRefresh();

        /// <summary>A handler for tab save</summary>
        void HandleSave();

        /// <summary>A handler for tab load</summary>
        void HandleLoad();

        /// <summary>A tab associated with this presenter</summary>
        ITabStripItem Tab { get; set; }

        /// <summary>Checks for presenters' equality</summary>
        bool IsEqualTo(ITabPresenter presenter);

        /// <summary>Checks whether tab's contents were modified</summary>
        bool IsDirty { get; }

        /// <summary>Gets caption for a tab</summary>
        string GetCaption();

        //bool BelongsToServer(ServerDescriptor serverDescriptor);

        void WaitTillTheEndOfTheWork();
        void HandleLock();
        void HandleUnLock();
    }
}