using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Shell.Proxies.Interfaces;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using StructureMap;

namespace Nuwanda.TfsSpotlight.Shell.Proxies
{
    [Author("Bartomiej Legid")]
    [Pluggable("TabStripPresenter")]
    public class TabStripPresenter : ITabStripPresenter
    {
        private ITabStrip View { get; set; }
        private ITabsService TabsService { get; set; }
        private IMenuService MenuService { get; set; }

        public TabStripPresenter(ITabStrip view, ITabsService tabsService, IMenuService menuService)
        {
            this.View = view;
            this.TabsService = tabsService;
            this.MenuService = menuService;
        }

        public void Initialize()
        {
            this.View.AttachPresenter(this);
        }

        public void HandleTabClosing(ITabStripItem tab)
        {
            this.TabsService.HandleTabRemoved(tab.Presenter);
            this.MenuService.InvalidateCommands(this);
        }

        public void HandleSelectedTabChanged(ITabStripItem tab)
        {
            this.TabsService.HandleSelectedTabHanged(tab.Presenter);
            this.MenuService.InvalidateCommands(this);
        }

        public void HandleTabScrolled(int direction)
        {
            var nextIndex = this.View.SelectedTabIndex + direction;
            var tabsCount = this.View.Count;

            if (nextIndex < 0)
            {
                nextIndex += tabsCount;
            }
            nextIndex %= tabsCount;

            this.View.SelectTab(nextIndex);
        }
    }
}