using System;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Data;
using Nuwanda.TfsSpotlight.Shell.Proxies.Interfaces;
using Nuwanda.TfsSpotlight.Shell.Services;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using StructureMap;

namespace Nuwanda.TfsSpotlight.Commands
{
    [Author("Bartomiej Legid")]
    [Pluggable(CommandsNames.CopyWorkItemIdToClipboard)]
    public class CopyWorkItemIdToClipboardCommand : ICommand
    {
        private ITabsService TabsService { get; set; }
        private IClipboardService ClipboardService { get; set; }
        private IMonitService MonitService { get; set; }

        public CopyWorkItemIdToClipboardCommand(
            ITabsService tabsService, IClipboardService clipboardService, IMonitService monitService)
        {
            this.TabsService = tabsService;
            this.ClipboardService = clipboardService;
            MonitService = monitService;
        }

        public void Execute()
        {
            var presenter = this.TabsService.GetActiveTab<ITabPresenter>();
            if (presenter == null || !(presenter is IWorkItemInfo)) return;

            var workItemInfo = presenter as IWorkItemInfo;
            try
            {
                this.ClipboardService.SetText(workItemInfo.WorkItemId.ToString());
            }
            catch(InvalidOperationException)
            {
                this.MonitService.ShowStatusMessage(Messages.MsgErrorWhilePuttingDataToTheClipboard, MessageSeverity.Error);
            }
        }
    }
}