using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Shell.Services;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using Nuwanda.TfsSpotlight.TFS;
using Nuwanda.TfsSpotlight.Views.GoToWorkItem;
using Nuwanda.TfsSpotlight.Views.WorkItems;
using StructureMap;
using ObjectFactory=StructureMap.ObjectFactory;

namespace Nuwanda.TfsSpotlight.Commands
{
    [Author("Bartomiej Legid")]
    [Pluggable(CommandsNames.GoToWorkItem)]
    public class GoToWorkItemCommand : ICommand
    {
        private IGoToWorkItemPresenter GoToWorkItemPresenter { get; set; }
        private ITabsService TabsService { get; set; }
        private IMonitService MonitService { get; set; }
        private ILogger Logger { get; set; }
        private ITfsServerRepository TfsServerRepository { get; set; }

        public GoToWorkItemCommand(IGoToWorkItemPresenter goToWorkItemPresenter,
            ITabsService tabsService, IMonitService monitService,
            ILogger logger, ITfsServerRepository tfsServerRepository)
        {
            this.GoToWorkItemPresenter = goToWorkItemPresenter;
            this.TabsService = tabsService;
            this.MonitService = monitService;
            this.Logger = logger;
            this.TfsServerRepository = tfsServerRepository;
        }

        public void Execute()
        {
            if(this.GoToWorkItemPresenter.WorkItemId <= 0)
            {
                this.MonitService.ShowStatusMessage(Messages.MsgWorkItemIdIsNotCorrect, MessageSeverity.Warning);
                this.GoToWorkItemPresenter.Focus();
                return;
            }

            if(this.GoToWorkItemPresenter.SelectedServer == null)
            {
                this.Logger.WriteError("A server for go to work item command was not set up");
                return;
            }

            var serverManager = this.TfsServerRepository.GetServer(this.GoToWorkItemPresenter.SelectedServer);

            var presenter = ObjectFactory.With(serverManager).GetInstance<IStandaloneWorkItemPresenter>();
            presenter.Initialize(
                this.GoToWorkItemPresenter.WorkItemId,
                this.GoToWorkItemPresenter.SelectedServer);

            this.TabsService.OpenTab(presenter);
        }
    }
}