using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using Nuwanda.TfsSpotlight.TFS;
using Nuwanda.TfsSpotlight.Views.Query;
using Nuwanda.TfsSpotlight.Views.WorkItems;
using StructureMap;

namespace Nuwanda.TfsSpotlight.Commands
{
    [Author("Bartomiej Legid")]
    [Pluggable(CommandsNames.OpenWorkItem)]
    public class OpenWorkItemCommand : ICommand
    {
        private ITabsService TabsService { get; set; }
        private ITfsServerRepository TfsServerRepository { get; set; }

        public OpenWorkItemCommand(ITabsService tabsService, ITfsServerRepository tfsServerRepository)
        {
            this.TabsService = tabsService;
            this.TfsServerRepository = tfsServerRepository;
        }

        public void Execute()
        {
            var queryPresenter = this.TabsService.GetActiveTab<IQueryPresenter>();

            if (queryPresenter == null) return;

            var tfsServerManager = this.TfsServerRepository.GetServer(queryPresenter.Server);

            var presenter = ObjectFactory.With(tfsServerManager).GetInstance<IStandaloneWorkItemPresenter>();

            presenter.Initialize(
                queryPresenter.SelectedWorkItemId,
                queryPresenter.Server);

            this.TabsService.OpenTab(presenter);
        }
    }
}