using System;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Shell.Data;

namespace Nuwanda.TfsSpotlight.Data
{
    /// <summary>Stores all application settings</summary>
    [Author("Bartomiej Legid")]
    [Serializable]
    public class ApplicationSettings : BaseEntity<ApplicationSettings>
    {
        private ServersSettings _serversSettings;
        /// <summary>Settings concerning TFS servers</summary>
        public ServersSettings ServersSettings
        {
            get { return this._serversSettings; }
            set
            {
                 this._serversSettings = value;
                 if (this._serversSettings != null)
                     this._serversSettings.PropertyChanged += delegate { OnPropertyChanged("ServersSettings"); };
            }
        }

        /// <summary>Checks whether settings have been modified</summary>
        public override bool IsDirty
        {
            get { return this.ServersSettings.IsDirty || base.IsDirty; }
        }

        public override bool IsValid
        {
            get { return this.ServersSettings.IsValid && base.IsValid; }
        }

        /// <summary>Marks all settings as old (= not dirty)</summary>
        public override void MarkOld()
        {
            this.ServersSettings.MarkOld();
            base.MarkOld();
        }
    }
}