using Nuwanda.Common;
using System;
using Nuwanda.TfsSpotlight.Shell.Data;

namespace Nuwanda.TfsSpotlight.Data
{
    /// <summary>Stores settings concerning TFS servers</summary>
    [Author("Bartomiej Legid")]
    [Serializable]
    public class ServersSettings : BaseEntity<ServersSettings>
    {
        private readonly ServerDescriptorCollection _servers;
        /// <summary>Registered servers</summary>
        public ServerDescriptorCollection Servers
        {
            get { return _servers; }
        }

        /// <summary>Checks whether settings have been modified</summary>
        public override bool IsDirty
        {
            get { return this._servers.IsDirty || base.IsDirty; }
        }

        public override bool IsValid
        {
            get { return this.Servers.IsValid && base.IsValid; }
        }

        /// <summary>Marks settings as old (= not dirty)</summary>
        public override void MarkOld()
        {
            this._servers.MarkOld();
            base.MarkOld();
        }

        public ServersSettings()
        {
            this._servers = new ServerDescriptorCollection();
            this._servers.ListChanged += delegate { OnPropertyChanged("Servers"); };
        }
    }
}