using System.Windows.Forms;
using Nuwanda.Common;

namespace Nuwanda.Windows.Forms
{
    [Author("Bartomiej Legid")]
    public class ListBox : System.Windows.Forms.ListBox
    {
        public bool EnableHotTrack { get; set; }

        public ListBox()
        {
            this.DrawMode = DrawMode.OwnerDrawVariable;
        }

        protected override void OnSizeChanged(System.EventArgs e)
        {
            base.OnSizeChanged(e);
            this.Invalidate(true);
        }

        protected override void WndProc(ref Message m)
        {
            if (m.Msg == 0x0001)
            {
                this.CreateParams.ExStyle |= 0x02000000;
            }

            base.WndProc(ref m);
        }

        protected override void OnDrawItem(DrawItemEventArgs e)
        {
            if (e.Index < 0 || e.Index >= this.Items.Count) return;

            var item = this.Items[e.Index] as IListBoxItem;
            if (item == null) return;

            item.Draw(e);
        }

        protected override void OnMeasureItem(MeasureItemEventArgs e)
        {
            if (e.Index < 0 || e.Index >= this.Items.Count) return;

            var item = this.Items[e.Index] as IListBoxItem;
            if (item == null) return;

            var size = item.Measure(e);

            e.ItemHeight = size.Height;
            e.ItemWidth = size.Width != 0 ? size.Width : this.ClientSize.Width;
        }

        protected override void OnMouseDoubleClick(MouseEventArgs e)
        {
            var index = this.IndexFromPoint(e.Location);
            if (index < 0 || index >= this.Items.Count) return;

            var item = this.Items[index] as IListBoxItem;
            if(item == null) return;

            item.HandleDoubleClick(e);
        }

        protected override void OnMouseMove(MouseEventArgs e)
        {
            if(this.EnableHotTrack == false) return;

            var index = this.IndexFromPoint(e.Location);
            if(index  < 0 || index >= this.Items.Count) return;

            if(this.SelectedIndex == index) return;

            this.SetSelected(index, true);
        }
    }
}