using System;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Data;
using Nuwanda.TfsSpotlight.Services.Interfaces;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using StructureMap;

namespace Nuwanda.TfsSpotlight.Services
{
    [Author("Bartomiej Legid")]
    [Pluggable("HistoryService")]
    public class HistoryService : IHistoryService
    {
        private const string HistoryStoreFileName = "History.dat";

        private IIsolatedStorageService IsolatedStorageService { get; set; }
        private IMenuService MenuService { get; set; }

        public HistoryService(IIsolatedStorageService isolatedStorageService, IMenuService menuService)
        {
            this.IsolatedStorageService = isolatedStorageService;
            this.MenuService = menuService;
        }

        public History GetHistory()
        {
            return this.IsolatedStorageService.Load<History>(HistoryStoreFileName) ?? new History();
        }

        public void RegisterOperation(HistoryEntry operation)
        {
            if(operation == null)
                throw new ArgumentNullException("operation");

            var history = GetHistory();

            history.Entries.RemoveAll(entry => entry.Equals(operation));
            history.Entries.Insert(0, operation);

            this.IsolatedStorageService.Save(history, HistoryStoreFileName);
            this.MenuService.RunCommand(CommandsNames.RefreshHistory);
        }

        public void ClearHistory()
        {
            this.IsolatedStorageService.Save(new History(), HistoryStoreFileName);
            this.MenuService.RunCommand(CommandsNames.RefreshHistory);
        }
    }
}