using System.Collections.Generic;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Data;
using StructureMap;
using StructureMap.Attributes;

namespace Nuwanda.TfsSpotlight.Services.Interfaces
{
    [Author("Bartomiej Legid")]
    [PluginFamily("UserSettingsService", Scope = InstanceScope.Singleton)]
    public interface IUserSettingsService
    {
        /// <summary>Gets defined servers</summary>
        /// <returns>A list of servers</returns>
        IList<ServerDescriptor> GetServers();

        /// <summary>Saves shell layout state. Data is stored per user in isolated storage</summary>
        /// <param name="layoutState">Layout state to save</param>
        void SaveShellLayoutState(ShellLayoutState layoutState);

        /// <summary>Loads slell layout state</summary>
        /// <returns>Loaded shell layout state. Returns null if no state found</returns>
        ShellLayoutState LoadShellLayoutState();

        /// <summary>Saves query layout state. Data is stored per user in isolated storage</summary>
        /// <param name="layoutState">Query layout to save</param>
        /// <param name="query">Query</param>
        void SaveQueryLayoutState(QueryLayoutState layoutState, QueryDescriptor query);

        /// <summary>Loads query layout state</summary>
        /// <param name="query">A query to load layout for</param>
        /// <returns>Loaded query layout state. Returns null if no state found</returns>
        QueryLayoutState LoadQueryLayoutState(QueryDescriptor query);

        /// <summary>Discovers TFS servers defined for Visual Studio</summary>
        List<ServerDescriptor> DiscoverServers();

        /// <summary>Retrieves application settings</summary>
        ApplicationSettings GetSettings();

        /// <summary>Saves application settings</summary>
        void SaveSettings(ApplicationSettings settings);

        void GetProjectConfiguration(ProjectDescriptor projectDescriptor);
        void SaveProjectConfiguration(ProjectDescriptor projectDescriptor);
    }
}