using System;
using System.Collections.Generic;
using Microsoft.TeamFoundation.Build.Proxy;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Data;
using StructureMap;

namespace Nuwanda.TfsSpotlight.TFS
{
    [Author("Bartomiej Legid")]
    [PluginFamily("TfsServerManager")]
    public interface ITfsServerManager
    {
        ServerDescriptor ServerDescriptor { get; set; }
        bool IsConnected { get; }
        bool IsWorking { get; }
        bool IsConnectionLost { get; }
        event EventHandler Connecting;
        event EventHandler Connected;
        void Connect();
        List<ProjectDescriptor> GetProjects();
        event EventHandler<ErrorEventArgs> Error;
        event EventHandler RetrievingProjects;
        event EventHandler ProjectsRetrieved;
        QueryResult RunQuery(QueryDescriptor query);
        void SaveWorkItems(List<WorkItem> workItems);
        WorkItem GetWorkItem(int workItemId);
        void Disconnect();
        event EventHandler Disconnecting;
        event EventHandler Disconnected;

        /// <exception cref="ConcurrentAccessException">Thrown when there was a concurrent access to the saved work item.</exception>
        /// <exception cref="WorkItemValidationException">Thrown when the saved work item is not valid.</exception>
        void SaveWorkItem(WorkItem workItem);

        event EventHandler ConnectionLost;
        void Reconnect();
        event EventHandler Reconnecting;
        event EventHandler Reconnected;
        void Refresh(WorkItem workItem);
        void ReportConnectionLost();
        List<BuildDataDescriptor> GetBuilds(BuildDescriptor buildDescriptor);
    }
}