using System;
using System.Collections.Generic;
using System.Linq;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Data;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using StructureMap;

namespace Nuwanda.TfsSpotlight.TFS
{
    [Author("Bartomiej Legid")]
    [Pluggable("TfsServerRepository")]
    public class TfsServerRepository : ITfsServerRepository
    {
        protected readonly Dictionary<string, ITfsServerManager> _managers;

        private IObjectFactory ObjectFactory { get; set; }

        public event EventHandler<ServerEventArgs> ServerConnected = delegate { };
        public event EventHandler<ServerEventArgs> ServerDisconnected = delegate { };
        public event EventHandler<ServerEventArgs> ServerProjectsLoaded = delegate { };

        public TfsServerRepository(IObjectFactory objectFactory)
        {
            this._managers = new Dictionary<string, ITfsServerManager>();
            this.ObjectFactory = objectFactory;
        }

        public ITfsServerManager GetServer(ServerDescriptor serverDecriptor)
        {
            if(serverDecriptor == null)
                throw  new ArgumentNullException("serverDecriptor");

            if (this._managers.ContainsKey(serverDecriptor.Url))
                return this._managers[serverDecriptor.Url];

            var manager = this.ObjectFactory.GetInstance<ITfsServerManager>();
            manager.ServerDescriptor = serverDecriptor;

            manager.Connected += 
                (sender, e) => this.ServerConnected(this, new ServerEventArgs(sender as ITfsServerManager));
            manager.Disconnected += 
                (sender, e) => this.ServerDisconnected(this, new ServerEventArgs(sender as ITfsServerManager));
            manager.ProjectsRetrieved +=
                (sender, e) => this.ServerProjectsLoaded(this, new ServerEventArgs(sender as ITfsServerManager));


            this._managers.Add(serverDecriptor.Url, manager);

            return manager;
        }

        public ITfsServerManager GetServer(string serverUrl)
        {
            return this._managers.ContainsKey(serverUrl) ? this._managers[serverUrl] : null;
        }

        public IEnumerable<ITfsServerManager> GetConnectedServers()
        {
            return from manager in this._managers.Values
                   where manager.IsConnected
                   select manager;
        }

        public IEnumerable<ITfsServerManager> GetAllServers()
        {
            return from manager in this._managers.Values
                   select manager;
        }
    }
}