using System;
using System.Drawing;
using System.Windows.Forms;
using ComponentFactory.Krypton.Toolkit;
using Nuwanda.Common;

namespace Nuwanda.TfsSpotlight.Views
{
    [Author("Bartomiej Legid")]
    public class BaseUserControl : UserControl, IBaseControl
    {
        private readonly KryptonHeader _monitBar;

        protected BaseUserControl()
        {
            _monitBar = new KryptonHeader();
            _monitBar.Dock = DockStyle.Top;
            _monitBar.Values.Heading = String.Empty;
            _monitBar.Values.Description = String.Empty;
            _monitBar.HeaderStyle = HeaderStyle.Form;

            var button = new ButtonSpecAny();
            button.Text = "Close";
            button.Click += this.CloseClickHandler;
            _monitBar.ButtonSpecs.Add(button);
        }

        private void CloseClickHandler(object sender, EventArgs e)
        {
            HideMessage();
        }

        public void HideMessage()
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new MethodInvoker(HideMessage));
                return;
            }

            this.Controls.Remove(this._monitBar);
        }

        public void ShowError(string message)
        {
            ShowMessage(message, Properties.Resources.Error);
        }

        public void ShowWarning(string message)
        {
            ShowMessage(message, Properties.Resources.Warning);
        }

        public void ShowInfo(string message)
        {
            ShowMessage(message, Properties.Resources.Information);
        }

        private void ShowMessage(string message, Image icon)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new MethodInvoker(() => ShowMessage(message, icon)));
                return;
            }

            this._monitBar.Values.Image = icon;
            this._monitBar.Values.Heading = message;

            if(this.Controls.Contains(this._monitBar) == false)
                this.Controls.Add(this._monitBar);
        }
    }
}