using System;
using System.Collections.Generic;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Data;
using Nuwanda.TfsSpotlight.Shell.Proxies.Interfaces;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using Nuwanda.TfsSpotlight.TFS;
using StructureMap;

namespace Nuwanda.TfsSpotlight.Views.Builds
{
    [Author("Bartomiej Legid")]
    [Pluggable("BuildsPresenter")]
    public class BuildsPresenter : TabPresenterBase<IBuildsView>, IBuildsPresenter, IServerAssociation
    {
        private ITfsServerManager _server;
        private ITfsServerRepository TfsServerRepository { get; set; }

        private BuildDescriptor BuildDescriptor { get; set; }

        public BuildsPresenter(IBuildsView view, IWorker worker, IMenuService menuService, ILogger logger,
            ITfsServerRepository tfsServerRepository) : base(view, worker, menuService, logger)
        {
            TfsServerRepository = tfsServerRepository;
        }

        public void Initialize(BuildDescriptor buildDescriptor)
        {
            this._server = this.TfsServerRepository.GetServer(buildDescriptor.Project.Server);
            this.BuildDescriptor = buildDescriptor;

            this.MenuService.RegisterCommandActivator(CommandsNames.Refresh)
                .Activator(RefreshCommandActivator).Context(this).Commit();
        }

        private bool RefreshCommandActivator()
        {
            return DefaultCommandActivator() && this._server.IsConnectionLost == false;
        }

        public override void HandleLoad()
        {
            HandleRefresh();
        }

        public override void HandleRefresh()
        {
            if (this.Worker.BeginWork() == false) return;
            this.InvalidateHandledCommands();
            this.View.HideMessage();

            try
            {
                this.View.StartProgress();
                this.View.DataSource = new List<BuildDataDescriptor>();
                this.Tab.SetCaption(GetCaption());
                this.View.DataSource = this._server.GetBuilds(this.BuildDescriptor);
                this.View.Deselect();
            }
            catch (BusinessLogicException ble)
            {
                this.View.ShowError(ble.Message);
            }
            catch (Exception e)
            {
                this.View.ShowError(String.Format(Messages.MsgInternalError, e.Message));
            }
            finally
            {
                this.View.StopProgress();

                this.Worker.EndWork();
                this.InvalidateHandledCommands();
            }
        }

        public override bool IsEqualTo(ITabPresenter presenter)
        {
            if(presenter is BuildsPresenter)
            {
                var buildsPresenter = presenter as BuildsPresenter;
                return buildsPresenter.BuildDescriptor.Equals(this.BuildDescriptor);
            }

            return false;
        }

        public override string GetCaption()
        {
            return this.BuildDescriptor.BuildType + " build";
        }

        #region Implementation of IServerAssociation

        public bool IsAssociatedWith(ServerDescriptor descriptor)
        {
            return descriptor.Equals(this._server.ServerDescriptor);
        }

        #endregion
    }
}