using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms.DataVisualization.Charting;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Data;

namespace Nuwanda.TfsSpotlight.Views.ChartView.ChartData
{
    [Author("Bartomiej Legid")]
    public class StringChartData
    {
        private SortedDictionary<string, int> Map { get; set; }
        private string FieldName { get; set; }

        public StringChartData()
        {
            this.Map = new SortedDictionary<string, int>();
        }

        public void CreateData(QueryResult result, string fieldName)
        {
            this.FieldName = fieldName;

            foreach (WorkItem workItem in result.Result)
            {
                var fieldValue = workItem.Fields[fieldName].Value;
                if (fieldValue == null) continue;

                if (this.Map.ContainsKey(fieldValue.ToString()) == false)
                {
                    this.Map.Add(fieldValue.ToString(), 1);
                }
                else
                {
                    this.Map[fieldValue.ToString()]++;
                }
            }
        }

        public Series GetData(ChartOptions options)
        {
            var series = new Series(this.FieldName);
            series.BorderColor = Color.Black;
            series.ChartType = options.ChartType;
            series["PieDrawingStyle"] = "Concave";
            series["PieLabelStyle"] = "Outside";

            
            var valueSort = new List<DataPoint>();
            foreach (var pair in this.Map)
            {
                var label = String.Format("{0} ({1})", pair.Key, pair.Value);
                var point = new DataPoint {AxisLabel = label, Name = pair.Key};
                point.SetValueXY(label, pair.Value);
                point.Label = label;
                point.ToolTip = label;
                valueSort.Add(point);
            }

            if(options.SortByValueCount)
            {
                valueSort.Sort(new DataPointValueComparer());
            }
            
            foreach (var dataPoint in valueSort)
            {
                series.Points.Add(dataPoint);
            }

            series.XValueType = ChartValueType.String;
            series.YValueType = ChartValueType.Int32;

            return series;
        }

        private class DataPointValueComparer : IComparer<DataPoint>
        {
            public int Compare(DataPoint x, DataPoint y)
            {
                return x.YValues[0].CompareTo(y.YValues[0]) * -1;
            }
        }
    }
}