using System.Windows.Forms;
using Nuwanda.Common;
using System;
using Nuwanda.TfsSpotlight.Shell;

namespace Nuwanda.TfsSpotlight.Views.GoToWorkItem
{
    [Author("Bartomiej Legid")]
    public class GoToWorkItemView : IGoToWorkItemView
    {
        private readonly ToolStripSplitButton _splitButton;
        private readonly ToolStripTextBox _textBox;
        private IGoToWorkItemPresenter _presenter;

        public ControlInvoker ToolStripInvoker { get; private set; }

        public GoToWorkItemView(ToolStripSplitButton splitButton, ToolStripTextBox textBox)
        {
            this._splitButton = splitButton;
            this._textBox = textBox;

            this._splitButton.Click += this.HandleSplitButtonClick;
            this._splitButton.DropDownOpening += this.HandleDropDownOpening;
            this._textBox.KeyDown += this.HandleTextBoxKeyDown;

            var menu = this._splitButton.DropDown as ToolStripDropDownMenu;
            if (menu != null)
            {
                menu.ShowCheckMargin = false;
                menu.ShowImageMargin = false;
            }
            
            this.ToolStripInvoker = new ControlInvoker(splitButton.Owner);
        }

        private void HandleDropDownOpening(object sender, EventArgs e)
        {
            this._presenter.HandleDropDownOpening();
        }

        public void AttachPresenter(IGoToWorkItemPresenter presenter)
        {
            this._presenter = presenter;
        }

        private void HandleSplitButtonClick(object sender, EventArgs e)
        {
            this._presenter.HandleExecute();
        }

        private void HandleTextBoxKeyDown(object sender, KeyEventArgs e)
        {
            this._presenter.HandleKeyDown(e.KeyCode);
        }

        public void ShowDropDown()
        {
            this.ToolStripInvoker.RunInControlsThread(
                () => this._splitButton.ShowDropDown());
        }

        public void HideDropDown()
        {
            this.ToolStripInvoker.RunInControlsThread(
                () => this._splitButton.HideDropDown());
        }

        public string GetTextBoxText()
        {
            return this.ToolStripInvoker.RunAndReturnInControlsThread(
                () => this._textBox.Text);
        }

        public void AddDropDownItem(ToolStripMenuItem item)
        {
            this.ToolStripInvoker.RunInControlsThread(
                () => this._splitButton.DropDownItems.Add(item));
        }

        public void RemoveDropDownItem(ToolStripMenuItem item)
        {
            this.ToolStripInvoker.RunInControlsThread(
                () => this._splitButton.DropDownItems.Remove(item));
        }

        public void ClearDropDownItems()
        {
            this.ToolStripInvoker.RunInControlsThread(
                () => this._splitButton.DropDownItems.Clear());
        }

        public void SetFocusToWorkItemIdTextBox()
        {
            this.ToolStripInvoker.RunInControlsThread(
                delegate
                    {
                        this._textBox.Focus();
                        this._textBox.SelectAll();
                    });
        }

        public void SetContent(string content)
        {
            this.ToolStripInvoker.RunInControlsThread(
                delegate
                {
                    this._textBox.Text = content;
                    this._textBox.SelectAll();
                });
        }
    }
}