using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using ComponentFactory.Krypton.Toolkit;
using Microsoft.Practices.EnterpriseLibrary.Validation.Validators;
using Nuwanda.TfsSpotlight.Data;
using Nuwanda.TfsSpotlight.Shell.Data;
using Nuwanda.Windows.Forms;

namespace Nuwanda.TfsSpotlight.Views.Options
{
    public class ServerSettingsItem : BaseEntity<ServerSettingsItem>, IListBoxItem
    {
        public ServerDescriptor ServerDescriptor { get; private set; }

        [StringLengthValidator(1, RangeBoundaryType.Inclusive, 0, RangeBoundaryType.Ignore, MessageTemplate = "Podaj nazw serwera")]
        public string Name
        {
            get { return this.ServerDescriptor.Name; }
            set
            {
                this.ServerDescriptor.Name = value;
                OnPropertyChanged("Name");
            }
        }

        public string Url
        {
            get { return this.ServerDescriptor.Url; }
            set
            {
                this.ServerDescriptor.Url = value;
                OnPropertyChanged("Url");
            }
        }

        public bool ConnectAtStartup
        {
            get { return this.ServerDescriptor.ConnectAtStartup; }
            set
            {
                this.ServerDescriptor.ConnectAtStartup = value;
                OnPropertyChanged("ConnectAtStartup");
            }
        }

        public ServerSettingsItem(ServerDescriptor serverDescriptor)
        {
            this.ServerDescriptor = serverDescriptor;
            Validate();
        }

        public Size Measure(MeasureItemEventArgs e)
        {
            return new Size(e.ItemWidth, 32);
        }

        public void Draw(DrawItemEventArgs e)
        {
            if (e.Bounds.Width == 0 || e.Bounds.Height == 0) return;

            var palette = KryptonManager.CurrentGlobalPalette;
            var style = PaletteBackStyle.ButtonListItem;

            if ((e.State & DrawItemState.Selected) == DrawItemState.Selected)
            {
                //using (var lgb = new LinearGradientBrush(e.Bounds, Color.WhiteSmoke, Color.LightSteelBlue, LinearGradientMode.Vertical))
                using (var lgb = new LinearGradientBrush(e.Bounds,
                    palette.GetBackColor1(style, PaletteState.Tracking),
                    palette.GetBackColor2(style, PaletteState.Tracking),
                    LinearGradientMode.Vertical))
                {
                    e.Graphics.FillRectangle(lgb, e.Bounds);
                }
            }
            else
            {
                using (var lgb = new LinearGradientBrush(e.Bounds,
                    palette.GetBackColor1(style, PaletteState.Normal),
                    palette.GetBackColor2(style, PaletteState.Normal),
                    LinearGradientMode.Vertical))
                {
                    e.Graphics.FillRectangle(lgb, e.Bounds);
                }
            }

            TextRenderer.DrawText(e.Graphics, this.Name, e.Font, new Point(e.Bounds.Left, e.Bounds.Top), Color.Black);
            TextRenderer.DrawText(e.Graphics, this.Url, e.Font, new Point(e.Bounds.Left, e.Bounds.Top + 15), Color.Black);
            e.Graphics.DrawLine(Pens.Black, new Point(e.Bounds.Left, e.Bounds.Bottom - 1), new Point(e.Bounds.Right, e.Bounds.Bottom - 1));

            if (this.ConnectAtStartup)
            {
                e.Graphics.DrawImage(Properties.Resources.ConnectAtStartupOn, e.Bounds.Right - 20, e.Bounds.Top + 2);
            }
            else
            {
                e.Graphics.DrawImage(Properties.Resources.ConnectAtStartupOff, e.Bounds.Right - 20 + 1, e.Bounds.Top + 2 + 1);
            }
        }

        public void HandleDoubleClick(MouseEventArgs e)
        {
            
        }
    }
}