using System;
using System.Windows.Forms;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Views.Project;
using Nuwanda.TfsSpotlight.Views.Servers;
using StructureMap;

namespace Nuwanda.TfsSpotlight.Views.Server
{
    [Author("Bartomiej Legid")]
    [Pluggable("ServerView")]
    public class ServerView : TreeNode, IServerView
    {
        private readonly TreeNode _infoNode;
        public IServerPresenter Presenter { get; private set; }

        public ServerView()
        {
            this.ImageIndex = (int)ServersNodeIcon.ServerNotConnected;
            this.SelectedImageIndex = (int)ServersNodeIcon.ServerNotConnected;

            this._infoNode = new TreeNode();
        }

        public void AttachPresenter(IServerPresenter presenter)
        {
            this.Presenter = presenter;
        }

        public void ClearProjects()
        {
            if(this.TreeView == null) return;

            if(this.TreeView.InvokeRequired)
            {
                this.TreeView.Invoke(new Action(this.ClearProjects));
                return;
            }

            if(this.FirstNode == this._infoNode)
            {
                this.Nodes.Clear();
                this.Nodes.Add(this._infoNode);
            }
            else
            {
                this.Nodes.Clear();
            }
        }

        public void ShowInfo(string message, ServersNodeIcon icon)
        {
            if (this.TreeView != null && this.TreeView.InvokeRequired)
            {
                this.TreeView.Invoke(new Action(() => this.ShowInfo(message, icon)));
                return;
            }

            if(this.FirstNode != this._infoNode)
            {
                this.Nodes.Insert(0, this._infoNode);
            }

            this._infoNode.Text = message;
            this._infoNode.ImageIndex = (int) icon;
            this._infoNode.SelectedImageIndex = (int) icon;
            this.Expand();
        }

        public void HideInfo()
        {
            if (this.TreeView != null && this.TreeView.InvokeRequired)
            {
                this.TreeView.Invoke(new Action(this.HideInfo));
                return;
            }

            if(this.FirstNode == this._infoNode)
            {
                this.Nodes.Remove(this._infoNode);
            }
        }

        public void SetIcon(ServersNodeIcon icon)
        {
            if (this.TreeView != null && this.TreeView.InvokeRequired)
            {
                this.TreeView.Invoke(new Action(() => this.SetIcon(icon)));
                return;
            }

            this.ImageIndex = (int) icon;
            this.SelectedImageIndex = (int)icon;
        }

        public void AddProject(IProjectView projectView)
        {
            if (this.TreeView != null && this.TreeView.InvokeRequired)
            {
                this.TreeView.Invoke(new Action(() => this.AddProject(projectView)));
                return;
            }

            if(projectView is TreeNode)
                this.Nodes.Add(projectView as TreeNode);
        }

        public void SetText(string name)
        {
            if (this.TreeView != null && this.TreeView.InvokeRequired)
            {
                this.TreeView.Invoke(new MethodInvoker(() => SetText(name)));
                return;
            }

            this.Text = name;
        }

        public ContextMenuStrip ContextMenu
        {
            set { this.ContextMenuStrip = value; }
        }
    }
}