using System;
using System.Drawing;
using System.Windows.Forms;
using ComponentFactory.Krypton.Toolkit;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Data;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using Nuwanda.Windows.Forms;
using StructureMap;

namespace Nuwanda.TfsSpotlight.Views.WorkItemHistory
{
    [Author("Bartomiej Legid")]
    public class HistoryEntryItem : IListBoxItem
    {
        public HistoryEntry HistoryEntry { get; private set; }
        private const int HEIGHT = 53;

        public HistoryEntryItem(HistoryEntry historyEntry)
        {
            HistoryEntry = historyEntry;
        }

        public Size Measure(MeasureItemEventArgs e)
        {
            return new Size(e.ItemWidth, HEIGHT);
        }

        public void Draw(DrawItemEventArgs e)
        {
            var bounds = e.Bounds;
            if (bounds.Width == 0 || bounds.Height == 0) return;

            var spliterDistance = 75;
            var splitterWidth = 0;

            var palette = KryptonManager.CurrentGlobalPalette;
            var style = PaletteBackStyle.ButtonListItem;
            var isSelected = (e.State & DrawItemState.Selected) == DrawItemState.Selected;
            var paletteState = isSelected ? PaletteState.Tracking : PaletteState.Normal;
            var drawInfo = bounds.Width > 2 * spliterDistance;

            var bolderFont = new Font(e.Font.FontFamily, e.Font.Size + 0.1f);
            
            using (var backgroundBriush = new SolidBrush(Color.White))
            {
                e.Graphics.FillRectangle(backgroundBriush, bounds);
            }

            bounds = new Rectangle(bounds.X + 3, bounds.Y + 3, bounds.Width - 3 - 3, bounds.Height - 3 - 2);
            
            var infoBounds = new Rectangle(bounds.X, bounds.Y, spliterDistance, bounds.Height);
            var titleBounds = new Rectangle(bounds.X + spliterDistance + splitterWidth, bounds.Y, bounds.Width - spliterDistance - splitterWidth, bounds.Height);
            titleBounds.Inflate(1, 1);

            if (drawInfo)
            {
                using (var infoBackgroundBriush = new Pen(palette.GetBackColor1(style, paletteState), 2.0f))
                {
                    e.Graphics.DrawRectangle(infoBackgroundBriush, infoBounds);
                }

                TextRenderer.DrawText(e.Graphics, String.Format("#{1}{0}{2}", Environment.NewLine, this.HistoryEntry.Id, this.HistoryEntry.ModificationDate),
                                      bolderFont, infoBounds, Color.Black, TextFormatFlags.Top | TextFormatFlags.Right | TextFormatFlags.WordBreak);
            }
            else
            {
                titleBounds = bounds;
            }


            using (var titleBackgroundBriush = new SolidBrush(palette.GetBackColor1(style, paletteState)))
            {
                e.Graphics.FillRectangle(titleBackgroundBriush, titleBounds);
            }

            titleBounds.Inflate(-1, -1);

            TextRenderer.DrawText(e.Graphics, String.Format("{0}", this.HistoryEntry.Title),
                                  bolderFont, titleBounds, Color.Black, TextFormatFlags.Top | TextFormatFlags.Left | TextFormatFlags.WordBreak);
        }

        public void HandleDoubleClick(MouseEventArgs e)
        {
            var menuService = ObjectFactory.GetInstance<IMenuService>();
            menuService.RunCommand(CommandsNames.OpenWorkItemFromHistory);
        }
    }
}