using System;
using System.Windows.Forms;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Data;
using Nuwanda.TfsSpotlight.Services.Interfaces;
using Nuwanda.TfsSpotlight.Shell.Services;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using Nuwanda.TfsSpotlight.TFS;
using Nuwanda.TfsSpotlight.Views.WorkItems;
using StructureMap;
using ObjectFactory=StructureMap.ObjectFactory;

namespace Nuwanda.TfsSpotlight.Views.WorkItemHistory
{
    [Author("Bartomiej Legid")]
    [Pluggable("HistoryPresenter")]
    public class HistoryPresenter : IHistoryPresenter
    {
        private History _dataSource;

        private IHistoryService HistoryService { get; set; }
        private ITfsServerRepository TfsServerRepository { get; set; }
        private ITabsService TabsService { get; set; }
        private IMonitService MonitService { get; set; }

        public IHistoryView View { get; private set; }
        public Control ViewConrol
        {
            get { return this.View as Control; }
        }

        public HistoryPresenter(
            IHistoryView view, IHistoryService historyService,
            ITfsServerRepository tfsServerRepository, ITabsService tabsService,
            IMonitService monitService)
        {
            View = view;
            HistoryService = historyService;
            TfsServerRepository = tfsServerRepository;
            TabsService = tabsService;
            MonitService = monitService;
        }

        public void Initialize()
        {
            this._dataSource = this.HistoryService.GetHistory();
            this.View.DataSource = this._dataSource;
        }

        public void HandleRefresh()
        {
            Initialize();
        }

        public void HandleOpen()
        {
            var entry = this.View.GetSelectedItem();
            if(entry == null) return;

            var tfsServerManager = this.TfsServerRepository.GetServer(entry.ServerUrl);
            if (tfsServerManager == null || tfsServerManager.IsConnected == false)
            {
                this.MonitService.ShowStatusMessage(String.Format("You're not connected to {0}.", entry.ServerUrl), MessageSeverity.Warning);
                return;
            }

            var presenter = ObjectFactory.With(tfsServerManager).GetInstance<IStandaloneWorkItemPresenter>();
            presenter.Initialize(entry.Id, tfsServerManager.ServerDescriptor);
            this.TabsService.OpenTab(presenter);
        }
    }
}