using System;
using System.Drawing;
using System.Windows.Forms;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Data;
using StructureMap;
using ListBox=Nuwanda.Windows.Forms.ListBox;

namespace Nuwanda.TfsSpotlight.Views.WorkItemHistory
{
    [Author("Bartomiej Legid")]
    [Pluggable("HistoryView")]
    public class HistoryView : ListBox, IHistoryView
    {
        public HistoryView()
        {
            this.BorderStyle = BorderStyle.None;
            this.BackColor = Color.White;
            this.EnableHotTrack = true;
            this.Cursor = Cursors.Hand;
        }

        public new History DataSource
        {
            set { SetDataSource(value); }
        }

        public HistoryEntry GetSelectedItem()
        {
            if(this.InvokeRequired)
            {
                return (HistoryEntry) this.Invoke(new Func<HistoryEntry>(GetSelectedItem));
            }

            var selected = this.SelectedItem as HistoryEntryItem;
            return selected == null ? null : selected.HistoryEntry;
        }

        private void SetDataSource(History dataSource)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new Action(() => this.SetDataSource(dataSource)));
                return;
            }

            base.DataSource = null;
            base.DataSource = dataSource.Entries.ConvertAll(entry => new HistoryEntryItem(entry));
        }
    }
}