using System;
using Nuwanda.TfsSpotlight.Data;
using Nuwanda.Common;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using Nuwanda.TfsSpotlight.Shell.Proxies.Interfaces;
using Nuwanda.TfsSpotlight.Shell.Services;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using Nuwanda.TfsSpotlight.TFS;

namespace Nuwanda.TfsSpotlight.Views.WorkItems
{
    /// <summary>Base class for presenters that drives work item's forms</summary>
    [Author("Bartomiej Legid")]
    public abstract class BaseWorkItemPresenter : TabPresenterBase<IWorkItemView>, IServerAssociation
    {
        protected WorkItem _dataSource;
        protected ServerDescriptor _server;

        protected IMonitService MonitService { get; private set; }
        protected ITfsServerManager TfsServerManager { get; private set; }

        public override bool IsDirty
        {
            get { return this._dataSource != null ? this._dataSource.IsDirty : false; }
        }

        protected BaseWorkItemPresenter(
            IWorkItemView view, IWorker worker,
            IMonitService monitService, IMenuService menuService,
            ILogger logger, ITfsServerManager tfsServerManager)
            : base(view, worker, menuService, logger)
        {
            this.MonitService = monitService;
            this.TfsServerManager = tfsServerManager;
        }

        public override string GetCaption()
        {
            return String.Empty;
        }

        public override void HandleClose() {}

        public override void HandleSave()
        {
            try
            {
                this.View.SetCaption(Captions.BarWorkItemSaving);
                this.MonitService.ShowStatusMessage(String.Format(Messages.MsgSavingWorkItem, this._dataSource.Title), MessageSeverity.Saving);
                this.View.Disable();

                this.TfsServerManager.SaveWorkItem(this._dataSource);

                this.MonitService.ClearStatusMessage();
                HandleWorkItemDataChanged();
            }
            catch(BusinessLogicException ble)
            {
                this.View.ShowError(ble.Message);
                this.View.SetCaption(String.Format(Captions.BarWorkItemUnableToSave, this._dataSource.Title));
            }
            catch (Exception e)
            {
                this.View.ShowError(String.Format(Messages.MsgInternalError, e.Message));
            }
            finally
            {
                this.View.Enable();
            }
        }

        public override bool IsEqualTo(ITabPresenter presenter)
        {
            return this.Equals(presenter);
        }

        protected void SetBarCaption()
        {
            if (this._dataSource == null)
            {
                this.View.SetCaption("No work item...");
                return;
            }

            var caption = String.Format(Captions.BarWorkItemNormal, this._dataSource.Type.Name, this._dataSource.Title);

            if(this._dataSource.Id == 0)
            {
                caption = String.Format(Captions.BarWorkItemNew, caption);
            }
            else if (this._dataSource.IsDirty)
            {
                caption = String.Format(Captions.BarWorkItemPendingEdit, caption);
            }

            this.View.SetCaption(caption);
        }

        protected void HandleWorkItemFieldChanged(object sender, WorkItemEventArgs e)
        {
            if (e.Field == null) return;

            this.HandleWorkItemDataChanged();
            this.InvalidateHandledCommands();
        }

        public override bool BelongsToServer(ServerDescriptor serverDescriptor)
        {
            return this._server.Equals(serverDescriptor);
        }

        protected abstract void HandleWorkItemDataChanged();

        public bool IsAssociatedWith(ServerDescriptor descriptor)
        {
            return this._server.Equals(descriptor);
        }
    }
}