using System;
using System.Windows.Forms;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Commands;
using Nuwanda.TfsSpotlight.Data;
using Nuwanda.TfsSpotlight.Shell.Proxies.Interfaces;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using Rhino.Mocks;
using NUnit.Framework;

namespace TfsSpotlight.Tests.Commands
{
    [TestFixture]
    [Author("Bartomiej Legid")]
    public class CopyWorkItemIdToClipboardCommandTests
    {
        private MockRepository _mocks;
        private ITabsService _tabsServiceMock;
        private IClipboardService _clipboardServiceMock;
        private IMonitService _monitServiceMock;

        private ICommand _testSubject;

        [SetUp]
        public void SetUp()
        {
            this._mocks = new MockRepository();
            this._tabsServiceMock = this._mocks.StrictMock<ITabsService>();
            this._clipboardServiceMock = this._mocks.StrictMock<IClipboardService>();
            this._monitServiceMock = this._mocks.StrictMock<IMonitService>();

            this._testSubject = new CopyWorkItemIdToClipboardCommand(this._tabsServiceMock, this._clipboardServiceMock, this._monitServiceMock);
        }

        [Test]
        public void WhenAskedTo_RunCommand_And_ThereIsNoActiveTab_Should_NotCopy()
        {
            using (this._mocks.Record())
            {
                Expect.Call(this._tabsServiceMock.GetActiveTab<ITabPresenter>()).Return(null);
            }

            using (this._mocks.Playback())
            {
                this._testSubject.Execute();
            }
        }

        [Test]
        public void WhenAskedTo_RunCommand_And_ThereIsActiveTab_And_ItsNotIWorkItemInfo_Should_NotCopy()
        {
            var tabPresenter = this._mocks.StrictMock<ITabPresenter>();

            using (this._mocks.Record())
            {
                Expect.Call(this._tabsServiceMock.GetActiveTab<ITabPresenter>()).Return(tabPresenter);
            }

            using (this._mocks.Playback())
            {
                this._testSubject.Execute();
            }
        }

        [Test]
        public void WhenAskedTo_RunCommand_And_ThereIsActiveTab_And_ItsAIWorkItemInfo_Should_Copy()
        {
            var tabPresenter = new WorkItemInfo();

            using (this._mocks.Record())
            {
                Expect.Call(this._tabsServiceMock.GetActiveTab<ITabPresenter>()).Return(tabPresenter);
                this._clipboardServiceMock.SetText("1");
            }

            using (this._mocks.Playback())
            {
                this._testSubject.Execute();
            }
        }

        private class WorkItemInfo : IWorkItemInfo, ITabPresenter
        {
            #region ITabPresenter Members

            public Control Control
            {
                get { throw new System.NotImplementedException(); }
            }

            public bool IsWorking
            {
                get { throw new System.NotImplementedException(); }
            }

            public void HandleActivation()
            {
                throw new System.NotImplementedException();
            }

            public void HandleDeactivation()
            {
                throw new System.NotImplementedException();
            }

            public void HandleClose()
            {
                throw new System.NotImplementedException();
            }

            public void HandleRefresh()
            {
                throw new System.NotImplementedException();
            }

            public void HandleSave()
            {
                throw new System.NotImplementedException();
            }

            public void HandleLoad()
            {
                throw new System.NotImplementedException();
            }

            public ITabStripItem Tab
            {
                get { throw new System.NotImplementedException(); }
                set { throw new System.NotImplementedException(); }
            }

            public bool IsEqualTo(ITabPresenter presenter)
            {
                throw new System.NotImplementedException();
            }

            public bool IsDirty
            {
                get { throw new System.NotImplementedException(); }
            }

            public string GetCaption()
            {
                throw new System.NotImplementedException();
            }

            public bool BelongsToServer(ServerDescriptor serverDescriptor)
            {
                throw new System.NotImplementedException();
            }

            public void WaitTillTheEndOfTheWork()
            {
                throw new System.NotImplementedException();
            }

            public void HandleLock()
            {
                throw new NotImplementedException();
            }

            public void HandleUnLock()
            {
                throw new NotImplementedException();
            }

            #endregion

            #region IWorkItemInfo Members

            public int WorkItemId
            {
                get { return 1; }
            }

            #endregion
        }
    }
}