using NUnit.Framework;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Commands;
using Nuwanda.TfsSpotlight.Shell.Services;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using Nuwanda.TfsSpotlight.TFS;
using Nuwanda.TfsSpotlight.Views.GoToWorkItem;
using Rhino.Mocks;
using ILogger=Nuwanda.TfsSpotlight.Shell.Services.Interfaces.ILogger;

namespace TfsSpotlight.Tests.Commands
{
    [Author("Bartomiej Legid")]
    [TestFixture]
    public class GoToWorkItemCommandTests
    {
        private MockRepository _mocks;
        private IGoToWorkItemPresenter _goToWorkItemPresenterMock;
        private ITabsService _tabsServiceMock;
        private IMonitService _monitServiceMock;
        private ILogger _loggerMock;
        private ITfsServerRepository _tfsServerRepositoryMOck;

        private ICommand _testSubject;

        [SetUp]
        public void SetUp()
        {
            this._mocks = new MockRepository();
            this._goToWorkItemPresenterMock = this._mocks.StrictMock<IGoToWorkItemPresenter>();
            this._tabsServiceMock = this._mocks.StrictMock<ITabsService>();
            this._monitServiceMock = this._mocks.StrictMock<IMonitService>();
            this._loggerMock = this._mocks.StrictMock<ILogger>();
            this._tfsServerRepositoryMOck = this._mocks.StrictMock<ITfsServerRepository>();

            this._testSubject = new GoToWorkItemCommand(this._goToWorkItemPresenterMock,
                this._tabsServiceMock, this._monitServiceMock, this._loggerMock, this._tfsServerRepositoryMOck);
        }

        [Test]
        public void WhenAskedTo_Execute_And_ThereIsNoWorkItemIdSet_Should_NotifyTheUser_And_SetFocus_And_Return()
        {
            using (this._mocks.Record())
            {
                Expect.Call(this._goToWorkItemPresenterMock.WorkItemId).Return(-1);
                this._monitServiceMock.ShowStatusMessage(null, MessageSeverity.Warning);
                LastCall.IgnoreArguments();
                this._goToWorkItemPresenterMock.Focus();
            }

            using (this._mocks.Playback())
            {
                this._testSubject.Execute();
            }
        }

        [Test]
        public void WhenAskedTo_Execute_And_ThereIsNoServerSet_Should_LogThatFact_And_Return()
        {
            using (this._mocks.Record())
            {
                Expect.Call(this._goToWorkItemPresenterMock.WorkItemId).Return(4);
                Expect.Call(this._goToWorkItemPresenterMock.SelectedServer).Return(null);
                this._loggerMock.WriteError(null);
                LastCall.IgnoreArguments();
            }

            using (this._mocks.Playback())
            {
                this._testSubject.Execute();
            }
        }

        //[Test]
        //public void WhenAskedTo_Execute_And_ThereAreAllNeededParameters_Should_OpenStandaloneWorkItemView()
        //{
        //    var workItemId = 4;
        //    var server = ObjectMother.CreateNamedServerDescriptor();
        //    var serverManager = this._mocks.StrictMock<ITfsServerManager>();
        //    var workItemView = this._mocks.StrictMock<IStandaloneWorkItemPresenter>();

        //    using (this._mocks.Record())
        //    {
        //        Expect.Call(this._goToWorkItemPresenterMock.WorkItemId).Return(workItemId).Repeat.Twice();
        //        Expect.Call(this._goToWorkItemPresenterMock.SelectedServer).Return(server).Repeat.Twice();
        //        Expect.Call(this._objectFactoryMock.GetInstance<IStandaloneWorkItemPresenter>()).Return(workItemView);
        //        Expect.Call(this._tfsServerRepositoryMOck.GetServer(server)).Return(serverManager);
        //        workItemView.Initialize(workItemId, server);
        //        this._tabsServiceMock.OpenTab(workItemView);
        //    }

        //    using (this._mocks.Playback())
        //    {
        //        this._testSubject.Execute();
        //    }
        //}
    }
}