using NUnit.Framework;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Commands;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using Nuwanda.TfsSpotlight.Views.Options;
using Rhino.Mocks;

namespace TfsSpotlight.Tests.Commands
{
    [Author("Bartomiej Legid")]
    [TestFixture]
    public class OpenSettingsCommandTests
    {
        private MockRepository _mocks;
        private ITabsService _tabsServiceMock;
        private IObjectFactory _objectFactoryMock;

        private ICommand _testSubject;

        [SetUp]
        public void SetUp()
        {
            this._mocks = new MockRepository();
            this._tabsServiceMock = this._mocks.StrictMock<ITabsService>();
            this._objectFactoryMock = this._mocks.StrictMock<IObjectFactory>();

            this._testSubject = new OpenSettingsCommand(this._tabsServiceMock, this._objectFactoryMock);
        }

        [Test]
        public void WhenAskedTo_Execute_Should_OpenSettingsView()
        {
            var optionsPresenter = this._mocks.StrictMock<IOptionsPresenter>();

            using (this._mocks.Record())
            {
                Expect.Call(this._objectFactoryMock.GetInstance<IOptionsPresenter>()).Return(optionsPresenter);
                optionsPresenter.Initialize();
                this._tabsServiceMock.OpenTab(optionsPresenter);
            }

            using (this._mocks.Playback())
            {
                this._testSubject.Execute();
            }
        }
    }
}