using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Commands;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using Nuwanda.TfsSpotlight.TFS;
using Nuwanda.TfsSpotlight.Views.Query;
using Nuwanda.TfsSpotlight.Views.WorkItems;
using Rhino.Mocks;
using NUnit.Framework;

namespace TfsSpotlight.Tests.Commands
{
    [TestFixture]
    [Author("Bartomiej Legid")]
    public class OpenWorkItemCommandTests
    {
        private MockRepository _mocks;
        private ITabsService _tabsServiceMock;
        private ITfsServerRepository _tfsServerRepository;

        private ICommand _testSubject;

        [SetUp]
        public void SetUp()
        {
            this._mocks = new MockRepository();
            this._tabsServiceMock = this._mocks.StrictMock<ITabsService>();
            this._tfsServerRepository = this._mocks.StrictMock<ITfsServerRepository>();

            this._testSubject = new OpenWorkItemCommand(this._tabsServiceMock, this._tfsServerRepository);
        }

        [Test]
        public void WhenAskedTo_RunCommand_AndActiveTabInNotQueryPresenter_Should_DoNothing()
        {
            using (this._mocks.Record())
            {
                Expect.Call(this._tabsServiceMock.GetActiveTab<IQueryPresenter>()).Return(null);
            }

            using (this._mocks.Playback())
            {
                this._testSubject.Execute();
            }
        }

        [Test]
        public void WhenAskedTo_RunCommand_AndActiveTabInQueryPresenter_Should_OpenStandaloneWorkItemView()
        {
            var queryPresenter = this._mocks.StrictMock<IQueryPresenter>();
            var standaloneWorkItemPresenter = this._mocks.StrictMock<IStandaloneWorkItemPresenter>();
            var workItemId = 1;
            var server = ObjectMother.CreateNamedServerDescriptor();

            using (this._mocks.Record())
            {
                Expect.Call(this._tabsServiceMock.GetActiveTab<IQueryPresenter>()).Return(queryPresenter);
                Expect.Call(queryPresenter.SelectedWorkItemId).Return(workItemId);
                Expect.Call(queryPresenter.Server).Return(server);

                standaloneWorkItemPresenter.Initialize(workItemId, server);

                this._tabsServiceMock.OpenTab(standaloneWorkItemPresenter);
            }

            using (this._mocks.Playback())
            {
                this._testSubject.Execute();
            }
        }
    }
}