using NUnit.Framework;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Shell.Proxies.Interfaces;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using Rhino.Mocks;
using Nuwanda.TfsSpotlight.Commands;

namespace TfsSpotlight.Tests.Commands
{
    [Author("Bartomiej Legid")]
    [TestFixture]
    public class RefreshCommandTests
    {
        private MockRepository _mocks;
        private ITabsService _tabsServiceMock;

        private ICommand _commandUnderTest;

        [SetUp]
        public void SetUp()
        {
            this._mocks = new MockRepository();
            this._tabsServiceMock = this._mocks.StrictMock<ITabsService>();

            this._commandUnderTest = new RefreshCommand(this._tabsServiceMock);
        }

        [Test]
        public void WhenAskedTo_RunCommand_And_ThereIsNoActiveTab_Should_SouldDoNothing()
        {
            using (this._mocks.Record())
            {
                Expect.Call(this._tabsServiceMock.GetActiveTab<ITabPresenter>()).Return(null);
            }

            using (this._mocks.Playback())
            {
                this._commandUnderTest.Execute();
            }
        }

        [Test]
        public void WhenAskedTo_RunCommand_And_ThereIsActiveTab_Should_DelegateActionToTabPresenter()
        {
            var tabPresenter = this._mocks.StrictMock<ITabPresenter>();

            using (this._mocks.Record())
            {
                Expect.Call(this._tabsServiceMock.GetActiveTab<ITabPresenter>()).Return(tabPresenter);
                tabPresenter.HandleRefresh();
            }

            using (this._mocks.Playback())
            {
                this._commandUnderTest.Execute();
            }
        }
    }
}