using NUnit.Framework;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using Nuwanda.TfsSpotlight.Views.Query;
using Rhino.Mocks;
using Nuwanda.TfsSpotlight.Commands;

namespace TfsSpotlight.Tests.Commands
{
    [Author("Bartomiej Legid")]
    [TestFixture]
    public class SaveAllCommandTests
    {
        private MockRepository _mocks;
        private ITabsService _tabsServiceMock;

        private ICommand _testSubject;

        [SetUp]
        public void SetUp()
        {
            this._mocks = new MockRepository();
            this._tabsServiceMock = this._mocks.StrictMock<ITabsService>();

            this._testSubject = new SaveAllCommand(this._tabsServiceMock);
        }

        [Test]
        public void WhenAskedTo_RunCommand_And_ThereIsNoActiveTab_Or_ActiveTabIsNotQueryTab_Should_SouldDoNothing()
        {
            using (this._mocks.Record())
            {
                Expect.Call(this._tabsServiceMock.GetActiveTab<IQueryPresenter>()).Return(null);
            }

            using (this._mocks.Playback())
            {
                this._testSubject.Execute();
            }
        }

        [Test]
        public void WhenAskedTo_RunCommand_And_ThereIsActiveCommand_Should_DelegateActionToQueryPresenter()
        {
            var queryPresenter = this._mocks.StrictMock<IQueryPresenter>();

            using (this._mocks.Record())
            {
                Expect.Call(this._tabsServiceMock.GetActiveTab<IQueryPresenter>()).Return(queryPresenter);
                queryPresenter.HandleSaveAll();
            }

            using (this._mocks.Playback())
            {
                this._testSubject.Execute();
            }
        }
    }
}