using NUnit.Framework;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Shell.Data;
using Rhino.Mocks;

namespace TfsSpotlight.Tests.Data
{
    [Author("Bartomiej Legid")]
    [TestFixture]
    public class BaseEntityCollectionTests
    {
        private MockRepository _mocks;

        [SetUp]
        public void SetUp()
        {
            this._mocks = new MockRepository();
        }

        [Test]
        public void WhenAskedTo_MarkOld_Should_MarkAllContainedEntitiesAsOld()
        {
            var entityMock = this._mocks.StrictMock<IBaseEntity>();
            var collectionUnderTest = new BaseEntityCollectionTestableStub{entityMock};

            using (this._mocks.Record())
            {
                LastCall.IgnoreArguments();//this is because collection implements IBindngList
                entityMock.MarkOld();
            }

            using (this._mocks.Playback())
            {
                entityMock.PropertyChanged += null;//this is because collection implements IBindngList
                collectionUnderTest.MarkOld();
            }
        }

        [Test]
        public void WhenAskedFor_IsDirty_And_ContainsDirtyEntity_Should_ReturnTrue()
        {
            var entityMock = this._mocks.StrictMock<IBaseEntity>();
            var collectionUnderTest = new BaseEntityCollectionTestableStub { entityMock };
            collectionUnderTest.Test_IsDirty = false;

            using (this._mocks.Record())
            {
                LastCall.IgnoreArguments();//this is because collection implements IBindngList
                Expect.Call(entityMock.IsDirty).Return(true);
            }

            using (this._mocks.Playback())
            {
                entityMock.PropertyChanged += null;//this is because collection implements IBindngList
                Assert.IsTrue(collectionUnderTest.IsDirty);
            }
        }

        [Test]
        public void WhenAskedFor_IsDirty_And_ContainsNoDirtyEntity_Should_ReturnFalse()
        {
            var entityMock = this._mocks.StrictMock<IBaseEntity>();
            var collectionUnderTest = new BaseEntityCollectionTestableStub { entityMock };
            collectionUnderTest.Test_IsDirty = false;

            using (this._mocks.Record())
            {
                LastCall.IgnoreArguments();//this is because collection implements IBindngList
                Expect.Call(entityMock.IsDirty).Return(false);
            }

            using (this._mocks.Playback())
            {
                entityMock.PropertyChanged += null;//this is because collection implements IBindngList
                Assert.IsFalse(collectionUnderTest.IsDirty);
            }
        }
    }
}