using NUnit.Framework;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Data;
using System.ComponentModel;
using System;

namespace TfsSpotlight.Tests.Data
{
    [Author("Bartomiej Legid")]
    [TestFixture]
    public class ServerDescriptorTests
    {
        [Test]
        public void When_NamePropertySet_Should_FirePropertyChangedEvent()
        {
            var server = new ServerDescriptor();
            var eventRaised = false;
            server.PropertyChanged +=
                delegate(object sender, PropertyChangedEventArgs e)
                    {
                        Assert.AreEqual("Name", e.PropertyName);
                        eventRaised = true;
                    };

            server.Name = "value";
            
            Assert.IsTrue(eventRaised);
        }

        [Test]
        public void When_ConnectAtStartupPropertySet_Should_FirePropertyChangedEvent()
        {
            var server = new ServerDescriptor();
            var eventRaised = false;
            server.PropertyChanged +=
                delegate(object sender, PropertyChangedEventArgs e)
                {
                    Assert.AreEqual("ConnectAtStartup", e.PropertyName);
                    eventRaised = true;
                };

            server.ConnectAtStartup = true;

            Assert.IsTrue(eventRaised);
            Assert.IsTrue(server.ConnectAtStartup);
        }

        [Test]
        public void When_InitializedViaConstructorParameters_Should_HaveProperValues()
        {
            var server = new ServerDescriptor("NAME", "URL");

            Assert.AreEqual("NAME", server.Name);
            Assert.AreEqual("URL", server.Url);
        }

        [Test]
        public void WhenAskedFor_Equality_And_PassedEntityWithSameUrl_Should_ReturnTrue()
        {
            var server = new ServerDescriptor("NAME", "URL");
            var cloneServer = new ServerDescriptor("NAME1", "URL");

            Assert.IsTrue(server.Equals(cloneServer));
        }

        [Test]
        public void WhenAskedFor_Equality_And_PassedEntityWithDifferentUrl_Should_ReturnFalse()
        {
            var server = new ServerDescriptor("NAME", "URL");
            var anotherServer = new ServerDescriptor("NAME1", "URL1");

            Assert.IsFalse(server.Equals(anotherServer));
        }

        [Test]
        public void WhenAskedFor_Equality_And_PassedonServerDescriptorEntity_Should_ReturnFalse()
        {
            var server = new ServerDescriptor("NAME", "URL");
            var otherObject = new Object();

            Assert.IsFalse(server.Equals(otherObject));
        }

        [Test]
        public void WhenAskedFor_HashCode_Should_UrlsHashCode()
        {
            var url = "URL";
            var server = new ServerDescriptor("NAME", url);

            Assert.AreEqual(url.GetHashCode(), server.GetHashCode());
        }
    }
}